/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */
 

public Action Command_Radio(int iClient, int iArgs)
{
	char szBaseSubCommand[32] = "";
	int iGenreFound = 0;

	if (g_cVarEnable.IntValue)
	{
		if (iArgs > 0)
		{
			GetCmdArg(1, szBaseSubCommand, sizeof(szBaseSubCommand));
			
			if (!strcmp(szBaseSubCommand, "version", false))
			{
				CReplyToCommand(iClient, "SourceMod Radio v%s by dubbeh - www.dubbeh.net", PLUGIN_VERSION);
				CReplyToCommand(iClient, "Powered by www.ShoutCast.com");
			}
			else if (g_bUseGenres && !strcmp(szBaseSubCommand, "genres", false))
			{
				if (iArgs > 1)
				{
					CReplyToCommand(iClient, "Listing available genres:");
					CReplyToCommand(iClient, CreateGenreList());
				}
				else if (g_mRadioBaseGenresMenu != null)
				{
					g_mRadioBaseGenresMenu.Display(iClient, MENU_TIME_FOREVER);
				}
				else
				{
					SetFailState("[SM-RADIO] Genres menu appears to be null. Please report this error to the author.");
				}
			}
			else if (!strcmp(szBaseSubCommand, "volume", false))
			{
				if (iArgs > 1)
				{
					char szVolume[64] = "";
					float fVolume = 0.1;
					
					GetCmdArg(2, szVolume, sizeof(szVolume));
					fVolume = StringToFloat(szVolume);
					
					if (fVolume > 0.01 && fVolume < 1.0)
					{
						char szStationID[32] = "";
						SetClientCookie(iClient, g_hCookieSavedVolume, szVolume);
						GetClientCookie(iClient, g_hCookieSavedStation, szStationID, sizeof(szStationID));
						LoadStation(iClient, szStationID);
					}
				}
			}
			else if (!strcmp(szBaseSubCommand, "help", false))
			{
				// TODO: Help Goes HERE
			}
			else if (!strcmp(szBaseSubCommand, "track", false) || !strcmp(szBaseSubCommand, "song", false))
			{
				if (g_iWebSocketType != -1)
					GetCurretTrack(iClient);
				else
					CReplyToCommand(iClient, "No websocket extenstion found to use the track query feature.");
			}
			// Check if a user did "!radio genre_name" and load the corresponding menu - Always do this last incase of overwriting potential commands with genre names
			else if (g_bUseGenres && ((iGenreFound = FindGenreIndexByName(szBaseSubCommand)) != -1))
			{
				RadioGenre genre = g_Genres.Items(iGenreFound);
				
				if (genre.IsValid && genre.m_hMenu != INVALID_HANDLE)
				{
					DisplayMenu(genre.m_hMenu, iClient, MENU_TIME_FOREVER);
				}
				else
				{
					char szGenreName[64] = "";
					genre.GetName(szGenreName, sizeof(szGenreName));
					SetFailState("[SM-RADIO] Genre \"%s\" using index \"%d\" appears to be invalid. Please report this error to the author.", szGenreName, genre.m_iID);
				}
			}
		}
		else if (g_mRadioBaseMenu != null)
		{
			g_mRadioBaseMenu.Display(iClient, MENU_TIME_FOREVER);
		}
		else
		{
			SetFailState("[SM-RADIO] Base menu appears to be null. Please report this error to the author.");
		}
	}
	
	return Plugin_Handled;
}

public Action Command_Browse(int iClient, int iArgs)
{
	if (g_cVarEnable.IntValue)
	{
		if (iArgs == 1)
		{
			char szWebsite[MAX_MOTD_URL_SIZE + 1] = "";
			
			GetCmdArg(1, szWebsite, sizeof(szWebsite));
			BrowseURL(iClient, szWebsite);
		}
		else
		{
			CReplyToCommand(iClient, "Invalid browse format");
			CReplyToCommand(iClient, "Usage: sm_browse \"www.website.com\" or !browse www.website.com");
		}
	}
	
	return Plugin_Handled;
}

public Action Command_RadioOff(int iClient, int iArgs)
{
	if (g_cVarEnable.IntValue)
		LoadOffPage(iClient);
	
	return Plugin_Handled;
}

